
/* Chrysalide - Outil d'analyse de fichiers binaires
 * edge.h - prototypes pour les liens entre les noeuds d'un graphique
 *
 * Copyright (C) 2013-2017 Cyrille Bagard
 *
 *  This file is part of Chrysalide.
 *
 *  Chrysalide is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  Chrysalide is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Chrysalide.  If not, see <http://www.gnu.org/licenses/>.
 */


#ifndef _GTKEXT_GRAPH_EDGE_H
#define _GTKEXT_GRAPH_EDGE_H


#include <glib-object.h>
#include <stdbool.h>
#include <gtk/gtk.h>



#define G_TYPE_GRAPH_EDGE               g_graph_edge_get_type()
#define G_GRAPH_EDGE(obj)               (G_TYPE_CHECK_INSTANCE_CAST((obj), G_TYPE_GRAPH_EDGE, GGraphEdge))
#define G_IS_GRAPH_EDGE(obj)            (G_TYPE_CHECK_INSTANCE_TYPE((obj), G_TYPE_GRAPH_EDGE))
#define G_GRAPH_EDGE_CLASS(klass)       (G_TYPE_CHECK_CLASS_CAST((klass), G_TYPE_GRAPH_EDGE, GGraphEdgeClass))
#define G_IS_GRAPH_EDGE_CLASS(klass)    (G_TYPE_CHECK_CLASS_TYPE((klass), G_TYPE_GRAPH_EDGE))
#define G_GRAPH_EDGE_GET_CLASS(obj)     (G_TYPE_INSTANCE_GET_CLASS((obj), G_TYPE_GRAPH_EDGE, GGraphEdgeClass))


/* Lien graphique entre deux noeuds graphiques (instance) */
typedef struct _GGraphEdge GGraphEdge;

/* Lien graphique entre deux noeuds graphiques (classe) */
typedef struct _GGraphEdgeClass GGraphEdgeClass;


/* Couleur de représentation */
typedef enum _EdgeColor
{
    EGC_DEFAULT,                            /* Noir, par défaut            */
    EGC_GREEN,                              /* Condition vérifiée          */
    EGC_RED,                                /* Condition non vérifiée      */
    EGC_BLUE,                               /* Boucle détectée             */
    EGC_DASHED_GRAY,                        /* Exception omniprésente      */

    EGC_COUNT

} EdgeColor;


/* Indique le type définit par la GLib pour les liens graphiques entre noeuds. */
GType g_graph_edge_get_type(void);

/* Etablit un lien graphique entre deux noeuds graphiques. */
GGraphEdge *_g_graph_edge_new(const GdkPoint *, const GdkPoint **, size_t, const GdkPoint *, EdgeColor);

#define g_graph_edge_new(start, y, end) \
    _g_graph_edge_new(start, (const GdkPoint *[]) { y }, 1, end, EGC_DEFAULT)

#define g_graph_edge_new_true(start, y, end) \
    _g_graph_edge_new(start, (const GdkPoint *[]) { y }, 1, end, EGC_GREEN)

#define g_graph_edge_new_false(start, y, end) \
    _g_graph_edge_new(start, (const GdkPoint *[]) { y }, 1, end, EGC_RED)

/* Fournit les abscisses des points extrèmes de la ligne. */
void g_graph_edge_get_x_borders(const GGraphEdge *, gint *, gint *);

/* Détermine les positions finales d'un lien graphique. */
void g_graph_edge_resolve(GGraphEdge *);

/* Opère un décalage du lien dans une direction donnée. */
void g_graph_edge_offset(GGraphEdge *, gint, gint);

/* Dessine les liens graphiques enregistrés dans le moteur. */
void g_graph_edge_draw(const GGraphEdge *, cairo_t *, bool);



#endif  /* _GTKEXT_GRAPH_EDGE_H */
